// The example shows a websocket client that is
// sending a text message to a connected server
// once in a second. The echoed response is
// printed to the console.

#INCLUDE "dcdialog.CH"

#define SERVERNAME  GetEnv("Computername" )
#define PORT        8100

PROCEDURE Main

LOCAL oEchoClient, GetList[0], oThread

//
// Instantiate the client side of the websocket
// connection and connect it to the server
//

// SomeVars()

oEchoClient := EchoClient():new( SERVERNAME,   ;
                                 "EchoServer", ;
                                 PORT )

IF .NOT. oEchoClient:connect()
  DCMSGBOX "Connection failed to server " + SERVERNAME + ':' + Alltrim(Str(PORT)) ;
     FONT '12.Lucida Console'
  RETURN
ENDIF

//
// As long as the connection is active
// handle all messages sent from the server
// in non blocking mode.
//

oThread := Thread():new()
Sleep(5)
oThread:start({||EchoClientLoop(oEchoClient)})

@ 0,0 DCPUSHBUTTON CAPTION 'Send Text' SIZE 35,1.8 ;
      ACTION {||SendData( oEchoClient, 'Hello from Server')} ;
      FONT '16.Lucida Console'

@ 2,0 DCPUSHBUTTON CAPTION 'Send Directory' SIZE 35,1.8 ;
      ACTION {||SendData( oEchoClient, Directory() )} ;
      FONT '16.Lucida Console'

@ 4,0 DCPUSHBUTTON CAPTION 'Send Object' SIZE 35,1.8 ;
      ACTION {||SendData( oEchoClient, SomeVars() )} ;
      FONT '16.Lucida Console'

@ 6,0 DCPUSHBUTTON CAPTION 'Send Code Blocks' SIZE 35,1.8 ;
      ACTION {||SendData( oEchoClient, CodeBlocks() )} ;
      FONT '16.Lucida Console'

DCREAD GUI TITLE 'WebSocket Client' FIT

RETURN

* -----------

PROC appsys ; RETURN

* -----------

STATIC FUNCTION EchoClientLoop( oEchoClient )

LOCAL lHandled

DO WHILE oEchoClient:isConnected()
  lHandled := oEchoClient:handleMessage( .F. )
  Sleep(5)
ENDDO

RETURN nil

* -----------

STATIC FUNCTION SendData( oEchoClient, xData )

LOCAL lStatus := .f.

IF !oEchoClient:isConnected()
  DCMSGBOX "Connection failed to server " + SERVERNAME + ':' + Alltrim(Str(PORT)) ;
     FONT '12.Lucida Console'
ELSEIF Valtype( xData ) $ 'MC'
  oEchoClient:sendText(xData)
  lStatus := .t.
ELSE
  oEchoClient:sendBinary(Var2Bin(xData))
  lStatus := .t.
ENDIF

WTF lStatus

RETURN nil


//
// The class echo client serves as client
// side of the EchoServer
//
CLASS EchoClient FROM WSCustomClient
  EXPORTED
    METHOD onText
    METHOD onConnect
    METHOD onDisconnect
    METHOD onBinary
ENDCLASS

//
// Implement the methods used by the
// EchoClient class
//
METHOD EchoClient:onText( cText )
  WTF cText COLOR GRA_CLR_DARKRED
RETURN self

METHOD EchoClient:onBinary( cBinary )
  WTF Bin2Var(cBinary) COLOR GRA_CLR_DARKRED
RETURN self

METHOD EchoClient:onConnect()
  WTF 'Connected'
RETURN self

METHOD EchoClient:onDisconnect()
  WTF 'DisConnected'
RETURN

* ------------

STATIC FUNCTION SomeVars()

LOCAL oVars := DataTransfer():new()

oVars:setVar('DIRECTORY',Directory())
oVars:setVar('DATE',Date())
oVars:setVar('TIME',Time())

RETURN oVars

* ------------

STATIC FUNCTION CodeBlocks()

RETURN { ;
  {||Directory()}, ;
  {||Date()}, ;
  {||Time()}, ;
  {||Os()} }


/*
 *  Derive from this class instead from
 */

CLASS WSCustomClient FROM WebSocketClient
   PROTECTED:
      VAR _AbortOnFailure
   EXPORTED:
      METHOD init()
      METHOD sendBinary()

      ACCESS METHOD getAbortOnFailure VAR AbortOnFailure
      ASSIGN METHOD setAbortOnFailure VAR AbortOnFailure
ENDCLASS

METHOD WSCustomClient:init()
   ::_AbortOnFailure := .T.
   SUPER
RETURN SELF

METHOD WSCustomClient:sendBinary( cBinary )
   LOCAL cType
   LOCAL cXOred
   LOCAL cMask

   wtf cBinary

   IF ::_AbortOnFailure

      cType := ValType( cBinary )
      IF .NOT. "C" == cType
         Exception():raiseParameterType( cBinary )
      ENDIF

   ENDIF

   IF ::_AbortOnFailure

      cMask  := RandomValue( 4 )
      cXOred := BXOR(cBinary, cMask)

      cType := ValType( cXOred )
      IF .NOT. "C" == cType
         Exception():raiseParameterType( cXOred )
      ENDIF

   ENDIF

   SUPER
RETURN self

METHOD WSCustomClient:getAbortOnFailure()
RETURN ::_AbortOnFailure

METHOD WSCustomClient:setAbortOnFailure( lAbortOnFailure )
   ::_AbortOnFailure := lAbortOnFailure
RETURN SELF



FUNCTION RandomValue( nLen )
RETURN RandomKey( nLen )



